"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExternalConfigCache = exports.InMemoryConfigCache = void 0;
const ProjectConfig_1 = require("./ProjectConfig.js");
const Utils_1 = require("./Utils.js");
class InMemoryConfigCache {
    constructor() {
        this.cachedConfig = ProjectConfig_1.ProjectConfig.empty;
    }
    set(_key, config) {
        this.cachedConfig = config;
    }
    get(_key) {
        return this.cachedConfig;
    }
    getInMemory() {
        return this.cachedConfig;
    }
}
exports.InMemoryConfigCache = InMemoryConfigCache;
class ExternalConfigCache {
    constructor(cache, logger) {
        this.cache = cache;
        this.logger = logger;
        this.cachedConfig = ProjectConfig_1.ProjectConfig.empty;
    }
    async set(key, config) {
        try {
            if (!config.isEmpty) {
                this.cachedSerializedConfig = ProjectConfig_1.ProjectConfig.serialize(config);
                this.cachedConfig = config;
            }
            else {
                this.cachedSerializedConfig = void 0;
                this.cachedConfig = config;
                return;
            }
            await this.cache.set(key, this.cachedSerializedConfig);
        }
        catch (err) {
            this.logger.configServiceCacheWriteError(err);
        }
    }
    updateCachedConfig(externalSerializedConfig) {
        if (externalSerializedConfig == null || externalSerializedConfig === this.cachedSerializedConfig) {
            return;
        }
        this.cachedConfig = ProjectConfig_1.ProjectConfig.deserialize(externalSerializedConfig);
        this.cachedSerializedConfig = externalSerializedConfig;
    }
    get(key) {
        try {
            const cacheGetResult = this.cache.get(key);
            if ((0, Utils_1.isPromiseLike)(cacheGetResult)) {
                return (async (cacheGetPromise) => {
                    try {
                        this.updateCachedConfig(await cacheGetPromise);
                    }
                    catch (err) {
                        this.logger.configServiceCacheReadError(err);
                    }
                    return this.cachedConfig;
                })(cacheGetResult);
            }
            this.updateCachedConfig(cacheGetResult);
        }
        catch (err) {
            this.logger.configServiceCacheReadError(err);
        }
        return Promise.resolve(this.cachedConfig);
    }
    getInMemory() {
        return this.cachedConfig;
    }
}
exports.ExternalConfigCache = ExternalConfigCache;
//# sourceMappingURL=ConfigCatCache.js.map