"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hooks = void 0;
const EventEmitter_1 = require("./EventEmitter.js");
const disconnectedEventEmitter = new EventEmitter_1.NullEventEmitter();
class Hooks {
    constructor(eventEmitter) {
        this.addListener = this.on;
        this.off = this.removeListener;
        this.eventEmitter = eventEmitter;
    }
    tryDisconnect() {
        const originalEventEmitter = this.eventEmitter;
        this.eventEmitter = disconnectedEventEmitter;
        return originalEventEmitter !== disconnectedEventEmitter;
    }
    on(eventName, listener) {
        this.eventEmitter.on(eventName, listener);
        return this;
    }
    once(eventName, listener) {
        this.eventEmitter.once(eventName, listener);
        return this;
    }
    removeListener(eventName, listener) {
        this.eventEmitter.removeListener(eventName, listener);
        return this;
    }
    removeAllListeners(eventName) {
        this.eventEmitter.removeAllListeners(eventName);
        return this;
    }
    listeners(eventName) {
        return this.eventEmitter.listeners(eventName);
    }
    listenerCount(eventName) {
        return this.eventEmitter.listenerCount(eventName);
    }
    eventNames() {
        return this.eventEmitter.eventNames();
    }
    emit(eventName, ...args) {
        return this.eventEmitter.emit(eventName, ...args);
    }
}
exports.Hooks = Hooks;
//# sourceMappingURL=Hooks.js.map