"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeHttpConfigFetcher = void 0;
const http = __importStar(require("http"));
const https = __importStar(require("https"));
const tunnel = __importStar(require("tunnel"));
const url_1 = require("url");
const ConfigCatLogger_1 = require("../ConfigCatLogger.js");
const ConfigFetcher_1 = require("../ConfigFetcher.js");
class NodeHttpConfigFetcher {
    constructor(options) {
        this.proxy = options === null || options === void 0 ? void 0 : options.proxy;
    }
    handleResponse(response, resolve, reject) {
        try {
            const { statusCode, statusMessage: reasonPhrase } = response;
            if (statusCode === 200) {
                const eTag = response.headers["etag"];
                const chunks = [];
                response
                    .on("data", chunk => chunks.push(chunk))
                    .on("end", () => {
                    try {
                        resolve({ statusCode, reasonPhrase, eTag, body: Buffer.concat(chunks).toString() });
                    }
                    catch (err) {
                        reject(err);
                    }
                })
                    .on("error", err => reject(new ConfigFetcher_1.FetchError("failure", err)));
            }
            else {
                response.resume();
                resolve({ statusCode, reasonPhrase });
            }
        }
        catch (err) {
            reject(err);
        }
    }
    fetchLogic(options, lastEtag) {
        return new Promise((resolve, reject) => {
            try {
                options.logger.debug("HttpConfigFetcher.fetchLogic() called.");
                const baseUrl = options.getUrl();
                const isBaseUrlSecure = baseUrl.startsWith("https");
                let agent;
                if (this.proxy) {
                    try {
                        const proxy = new url_1.URL(this.proxy);
                        let agentFactory;
                        if (proxy.protocol === "https:") {
                            agentFactory = isBaseUrlSecure ? tunnel.httpsOverHttps : tunnel.httpOverHttps;
                        }
                        else {
                            agentFactory = isBaseUrlSecure ? tunnel.httpsOverHttp : tunnel.httpOverHttp;
                        }
                        agent = agentFactory({
                            proxy: {
                                host: proxy.hostname,
                                port: proxy.port,
                                proxyAuth: (proxy.username && proxy.password) ? `${proxy.username}:${proxy.password}` : null
                            }
                        });
                    }
                    catch (err) {
                        options.logger.log(1, 0, ConfigCatLogger_1.FormattableLogMessage.from("PROXY") `Failed to parse \`options.proxy\`: '${this.proxy}'.`, err);
                    }
                }
                const headers = {
                    "User-Agent": options.clientVersion
                };
                if (lastEtag) {
                    headers["If-None-Match"] = lastEtag;
                }
                const requestOptions = {
                    agent,
                    headers,
                    timeout: options.requestTimeoutMs,
                };
                options.logger.debug(JSON.stringify(requestOptions));
                const request = (isBaseUrlSecure ? https : http).get(baseUrl, requestOptions, response => this.handleResponse(response, resolve, reject))
                    .on("timeout", () => {
                    try {
                        request.destroy();
                    }
                    finally {
                        reject(new ConfigFetcher_1.FetchError("timeout", options.requestTimeoutMs));
                    }
                })
                    .on("error", err => {
                    reject(new ConfigFetcher_1.FetchError("failure", err));
                })
                    .end();
            }
            catch (err) {
                reject(err);
            }
        });
    }
}
exports.NodeHttpConfigFetcher = NodeHttpConfigFetcher;
//# sourceMappingURL=NodeHttpConfigFetcher.js.map