"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FetchApiConfigFetcher = void 0;
const ConfigFetcher_1 = require("../ConfigFetcher.js");
class FetchApiConfigFetcher {
    async fetchLogic(options, lastEtag) {
        var _a, _b;
        const requestInit = { method: "GET" };
        let cleanup;
        if (typeof AbortController !== "undefined") {
            const controller = new AbortController();
            const timeoutId = setTimeout(() => controller.abort(), options.requestTimeoutMs);
            requestInit.signal = controller.signal;
            cleanup = () => clearTimeout(timeoutId);
        }
        else {
            cleanup = () => { };
        }
        try {
            let url = options.getUrl();
            if (lastEtag) {
                url += "&ccetag=" + encodeURIComponent(lastEtag);
            }
            const response = await fetch(url, requestInit);
            const { status: statusCode, statusText: reasonPhrase } = response;
            if (statusCode === 200) {
                const body = await response.text();
                const eTag = (_a = response.headers.get("ETag")) !== null && _a !== void 0 ? _a : void 0;
                return { statusCode, reasonPhrase, eTag, body };
            }
            else {
                return { statusCode, reasonPhrase };
            }
        }
        catch (err) {
            if (err instanceof DOMException && err.name === "AbortError") {
                if ((_b = requestInit.signal) === null || _b === void 0 ? void 0 : _b.aborted) {
                    throw new ConfigFetcher_1.FetchError("timeout", options.requestTimeoutMs);
                }
                else {
                    throw new ConfigFetcher_1.FetchError("abort");
                }
            }
            throw new ConfigFetcher_1.FetchError("failure", err);
        }
        finally {
            cleanup();
        }
    }
}
exports.FetchApiConfigFetcher = FetchApiConfigFetcher;
//# sourceMappingURL=FetchApiConfigFetcher.js.map