"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndexedDBConfigCache = void 0;
exports.getDBConnectionFactory = getDBConnectionFactory;
const ConfigCatCache_1 = require("../ConfigCatCache.js");
const OBJECT_STORE_NAME = "configCache";
class IndexedDBConfigCache {
    static tryGetFactory() {
        const dbConnectionFactory = getDBConnectionFactory();
        if (dbConnectionFactory) {
            return options => new ConfigCatCache_1.ExternalConfigCache(new IndexedDBConfigCache(dbConnectionFactory), options.logger);
        }
    }
    constructor(dbConnectionFactory) {
        this.dbConnectionFactory = dbConnectionFactory;
    }
    async set(key, value) {
        const db = await this.dbConnectionFactory();
        try {
            await new Promise((resolve, reject) => {
                const transaction = db.transaction(OBJECT_STORE_NAME, "readwrite");
                transaction.oncomplete = () => resolve();
                transaction.onerror = event => reject(event.target.error);
                const store = transaction.objectStore(OBJECT_STORE_NAME);
                store.put(value, key);
            });
        }
        finally {
            db.close();
        }
    }
    async get(key) {
        const db = await this.dbConnectionFactory();
        try {
            return await new Promise((resolve, reject) => {
                const transaction = db.transaction(OBJECT_STORE_NAME, "readonly");
                let value;
                transaction.oncomplete = () => resolve(value);
                transaction.onerror = event => reject(event.target.error);
                const store = transaction.objectStore(OBJECT_STORE_NAME);
                const storeRequest = store.get(key);
                storeRequest.onsuccess = event => value = event.target.result;
            });
        }
        finally {
            db.close();
        }
    }
}
exports.IndexedDBConfigCache = IndexedDBConfigCache;
function getDBConnectionFactory() {
    if (typeof indexedDB !== "undefined") {
        try {
            const dbConnectionFactory = () => new Promise((resolve, reject) => {
                const openRequest = indexedDB.open("@configcat/sdk");
                openRequest.onupgradeneeded = event => event.target.result.createObjectStore(OBJECT_STORE_NAME);
                openRequest.onsuccess = event => resolve(event.target.result);
                openRequest.onerror = event => reject(event.target.error);
            });
            dbConnectionFactory().then(db => db.close()).catch(() => { });
            return dbConnectionFactory;
        }
        catch (_a) { }
    }
}
//# sourceMappingURL=IndexedDBConfigCache.js.map