import { ProjectConfig } from "./ProjectConfig.js";
import { isPromiseLike } from "./Utils.js";
export class InMemoryConfigCache {
    constructor() {
        this.cachedConfig = ProjectConfig.empty;
    }
    set(_key, config) {
        this.cachedConfig = config;
    }
    get(_key) {
        return this.cachedConfig;
    }
    getInMemory() {
        return this.cachedConfig;
    }
}
export class ExternalConfigCache {
    constructor(cache, logger) {
        this.cache = cache;
        this.logger = logger;
        this.cachedConfig = ProjectConfig.empty;
    }
    async set(key, config) {
        try {
            if (!config.isEmpty) {
                this.cachedSerializedConfig = ProjectConfig.serialize(config);
                this.cachedConfig = config;
            }
            else {
                this.cachedSerializedConfig = void 0;
                this.cachedConfig = config;
                return;
            }
            await this.cache.set(key, this.cachedSerializedConfig);
        }
        catch (err) {
            this.logger.configServiceCacheWriteError(err);
        }
    }
    updateCachedConfig(externalSerializedConfig) {
        if (externalSerializedConfig == null || externalSerializedConfig === this.cachedSerializedConfig) {
            return;
        }
        this.cachedConfig = ProjectConfig.deserialize(externalSerializedConfig);
        this.cachedSerializedConfig = externalSerializedConfig;
    }
    get(key) {
        try {
            const cacheGetResult = this.cache.get(key);
            if (isPromiseLike(cacheGetResult)) {
                return (async (cacheGetPromise) => {
                    try {
                        this.updateCachedConfig(await cacheGetPromise);
                    }
                    catch (err) {
                        this.logger.configServiceCacheReadError(err);
                    }
                    return this.cachedConfig;
                })(cacheGetResult);
            }
            this.updateCachedConfig(cacheGetResult);
        }
        catch (err) {
            this.logger.configServiceCacheReadError(err);
        }
        return Promise.resolve(this.cachedConfig);
    }
    getInMemory() {
        return this.cachedConfig;
    }
}
//# sourceMappingURL=ConfigCatCache.js.map