export var FetchStatus;
(function (FetchStatus) {
    FetchStatus[FetchStatus["Fetched"] = 0] = "Fetched";
    FetchStatus[FetchStatus["NotModified"] = 1] = "NotModified";
    FetchStatus[FetchStatus["Errored"] = 2] = "Errored";
})(FetchStatus || (FetchStatus = {}));
export class FetchResult {
    constructor(status, config, errorMessage, errorException) {
        this.status = status;
        this.config = config;
        this.errorMessage = errorMessage;
        this.errorException = errorException;
    }
    static success(config) {
        return new FetchResult(0, config);
    }
    static notModified(config) {
        return new FetchResult(1, config);
    }
    static error(config, errorMessage, errorException) {
        return new FetchResult(2, config, errorMessage !== null && errorMessage !== void 0 ? errorMessage : "Unknown error.", errorException);
    }
}
export class FetchError extends Error {
    constructor(cause, ...args) {
        super(((cause, args) => {
            switch (cause) {
                case "abort":
                    return "Request was aborted.";
                case "timeout":
                    const [timeoutMs] = args;
                    return `Request timed out. Timeout value: ${timeoutMs}ms`;
                case "failure":
                    const [err] = args;
                    const message = "Request failed due to a network or protocol error.";
                    return err
                        ? message + " " + (err instanceof Error ? err.message : err + "")
                        : message;
            }
        })(cause, args));
        this.cause = cause;
        if (!(this instanceof FetchError)) {
            (Object.setPrototypeOf || ((o, proto) => o["__proto__"] = proto))(this, FetchError.prototype);
        }
        this.args = args;
    }
}
//# sourceMappingURL=ConfigFetcher.js.map