import { ConfigServiceBase } from "./ConfigServiceBase.js";
export class LazyLoadConfigService extends ConfigServiceBase {
    constructor(configFetcher, options) {
        super(configFetcher, options);
        this.cacheTimeToLiveMs = options.cacheTimeToLiveSeconds * 1000;
        const initialCacheSyncUp = this.syncUpWithCache();
        this.readyPromise = this.getReadyPromise(initialCacheSyncUp, async (initialCacheSyncUp) => this.getCacheState(await initialCacheSyncUp));
    }
    async getConfig() {
        this.options.logger.debug("LazyLoadConfigService.getConfig() called.");
        function logExpired(logger, appendix = "") {
            logger.debug(`LazyLoadConfigService.getConfig(): cache is empty or expired${appendix}.`);
        }
        let cachedConfig = await this.options.cache.get(this.cacheKey);
        if (cachedConfig.isExpired(this.cacheTimeToLiveMs)) {
            if (!this.isOffline) {
                logExpired(this.options.logger, ", calling refreshConfigCoreAsync()");
                [, cachedConfig] = await this.refreshConfigCoreAsync(cachedConfig);
            }
            else {
                logExpired(this.options.logger);
            }
            return cachedConfig;
        }
        this.options.logger.debug("LazyLoadConfigService.getConfig(): cache is valid, returning from cache.");
        return cachedConfig;
    }
    refreshConfigAsync() {
        this.options.logger.debug("LazyLoadConfigService.refreshConfigAsync() called.");
        return super.refreshConfigAsync();
    }
    getCacheState(cachedConfig) {
        if (cachedConfig.isEmpty) {
            return 0;
        }
        if (cachedConfig.isExpired(this.cacheTimeToLiveMs)) {
            return 2;
        }
        return 3;
    }
}
//# sourceMappingURL=LazyLoadConfigService.js.map