import { ExternalConfigCache } from "../ConfigCatCache.js";
export class LocalStorageConfigCache {
    static tryGetFactory() {
        const localStorage = getLocalStorage();
        if (localStorage) {
            return options => new ExternalConfigCache(new LocalStorageConfigCache(localStorage), options.logger);
        }
    }
    constructor(storage) {
        this.storage = storage;
    }
    set(key, value) {
        this.storage.setItem(key, toUtf8Base64(value));
    }
    get(key) {
        const configString = this.storage.getItem(key);
        if (configString) {
            return fromUtf8Base64(configString);
        }
    }
}
export function getLocalStorage() {
    if (typeof localStorage !== "undefined") {
        const testKey = "__configcat_localStorage_test";
        try {
            const storage = localStorage;
            storage.setItem(testKey, testKey);
            let retrievedItem;
            try {
                retrievedItem = storage.getItem(testKey);
            }
            finally {
                storage.removeItem(testKey);
            }
            if (retrievedItem === testKey) {
                return storage;
            }
        }
        catch (_a) { }
    }
}
export function toUtf8Base64(str) {
    str = encodeURIComponent(str);
    str = str.replace(/%([0-9A-F]{2})/g, (_, p1) => String.fromCharCode(parseInt(p1, 16)));
    return btoa(str);
}
export function fromUtf8Base64(str) {
    str = atob(str);
    str = str.replace(/[%\x80-\xFF]/g, m => "%" + m.charCodeAt(0).toString(16));
    return decodeURIComponent(str);
}
//# sourceMappingURL=LocalStorageConfigCache.js.map