import { FetchError } from "../ConfigFetcher.js";
export class XmlHttpRequestConfigFetcher {
    handleStateChange(httpRequest, resolve, reject) {
        var _a;
        try {
            if (httpRequest.readyState === 4) {
                const { status: statusCode, statusText: reasonPhrase } = httpRequest;
                if (statusCode === 200) {
                    const eTag = (_a = httpRequest.getResponseHeader("ETag")) !== null && _a !== void 0 ? _a : void 0;
                    resolve({ statusCode, reasonPhrase, eTag, body: httpRequest.responseText });
                }
                else if (statusCode) {
                    resolve({ statusCode, reasonPhrase });
                }
            }
        }
        catch (err) {
            reject(err);
        }
    }
    fetchLogic(options, lastEtag) {
        return new Promise((resolve, reject) => {
            try {
                options.logger.debug("HttpConfigFetcher.fetchLogic() called.");
                const httpRequest = new XMLHttpRequest();
                httpRequest.onreadystatechange = () => this.handleStateChange(httpRequest, resolve, reject);
                httpRequest.ontimeout = () => reject(new FetchError("timeout", options.requestTimeoutMs));
                httpRequest.onabort = () => reject(new FetchError("abort"));
                httpRequest.onerror = () => reject(new FetchError("failure"));
                let url = options.getUrl();
                if (lastEtag) {
                    url += "&ccetag=" + encodeURIComponent(lastEtag);
                }
                httpRequest.open("GET", url, true);
                httpRequest.timeout = options.requestTimeoutMs;
                httpRequest.send(null);
            }
            catch (err) {
                reject(err);
            }
        });
    }
}
//# sourceMappingURL=XmlHttpRequestConfigFetcher.js.map