import { ExternalConfigCache } from "../ConfigCatCache.js";
export class ChromeLocalStorageConfigCache {
    static tryGetFactory() {
        const localStorage = getChromeLocalStorage();
        if (localStorage) {
            return options => new ExternalConfigCache(new ChromeLocalStorageConfigCache(localStorage), options.logger);
        }
    }
    constructor(storage) {
        this.storage = storage;
    }
    async set(key, value) {
        await this.storage.set({ [key]: toUtf8Base64(value) });
    }
    async get(key) {
        const cacheObj = await this.storage.get(key);
        const configString = cacheObj[key];
        if (configString) {
            return fromUtf8Base64(configString);
        }
    }
}
export function getChromeLocalStorage() {
    var _a;
    if (typeof chrome !== "undefined") {
        return (_a = chrome.storage) === null || _a === void 0 ? void 0 : _a.local;
    }
}
export function toUtf8Base64(str) {
    str = encodeURIComponent(str);
    str = str.replace(/%([0-9A-F]{2})/g, (_, p1) => String.fromCharCode(parseInt(p1, 16)));
    return btoa(str);
}
export function fromUtf8Base64(str) {
    str = atob(str);
    str = str.replace(/[%\x80-\xFF]/g, m => "%" + m.charCodeAt(0).toString(16));
    return decodeURIComponent(str);
}
//# sourceMappingURL=ChromeLocalStorageConfigCache.js.map