import { ExternalConfigCache } from "../ConfigCatCache.js";
const CACHE_NAME = "@configcat/sdk";
export class CloudflareConfigCache {
    static tryGetFactory() {
        const cache = getCloudflareCache();
        if (cache) {
            return options => new ExternalConfigCache(new CloudflareConfigCache(cache), options.logger);
        }
    }
    constructor(cache) {
        this.cache = cache;
    }
    async set(key, value) {
        var _a;
        try {
            const cache = await ((_a = this.cache) !== null && _a !== void 0 ? _a : caches.open(CACHE_NAME));
            await cache.put(cacheUrlFor(key), new Response(value));
        }
        catch (err) {
            this.cache = void 0;
            throw err;
        }
    }
    async get(key) {
        var _a;
        try {
            const cache = await ((_a = this.cache) !== null && _a !== void 0 ? _a : caches.open(CACHE_NAME));
            const response = await cache.match(cacheUrlFor(key));
            if (response) {
                return await response.text();
            }
        }
        catch (err) {
            this.cache = void 0;
            throw err;
        }
    }
}
export function getCloudflareCache() {
    if (typeof caches !== "undefined") {
        try {
            return caches.open(CACHE_NAME);
        }
        catch (_a) { }
    }
}
function cacheUrlFor(key) {
    return `http://sdk-config-cache.configcat.com/${encodeURIComponent(key)}`;
}
//# sourceMappingURL=CloudflareConfigCache.js.map